@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-node-view.c,v 1.60 2005/04/19 11:14:41 chpe Exp $
+ *  $Id: ephy-node-view.c,v 1.62 2005/07/11 19:47:25 chpe Exp $
  */
 
 #include "config.h"
@@ -309,7 +309,8 @@ drag_motion_cb (GtkWidget *widget,
 				(node, view->priv->priority_prop_id);
 
 		if (priority != EPHY_NODE_VIEW_ALL_PRIORITY &&
-		    priority != EPHY_NODE_VIEW_SPECIAL_PRIORITY)
+		    priority != EPHY_NODE_VIEW_SPECIAL_PRIORITY &&
+		    ephy_node_get_is_drag_source (node))
 		{
 			action = context->suggested_action;
 		}
@@ -374,6 +375,10 @@ drag_data_received_cb (GtkWidget *widget,
 		return;
 	}	
 
+	/* appease GtkTreeView by preventing its drag_data_receive
+	* from being called */
+	g_signal_stop_emission_by_name (view, "drag_data_received");
+
 	if (!view->priv->have_drag_data)
 	{
 		view->priv->have_drag_data = TRUE;
@@ -388,16 +393,18 @@ drag_data_received_cb (GtkWidget *widget,
 		gboolean success = FALSE;
 		GtkTreePath *path;
 
-		gtk_tree_view_get_dest_row_at_pos
-			(GTK_TREE_VIEW (widget), x, y, &path, &pos);
-
-		g_return_if_fail (path != NULL);
+		if (gtk_tree_view_get_dest_row_at_pos
+			(GTK_TREE_VIEW (widget), x, y, &path, &pos) == FALSE)
+		{
+			return;
+		}
 
 		node = get_node_from_path (view, path);
+		if (node == NULL) return;
 
 		uris = gtk_selection_data_get_uris (selection_data);
 
-		if (uris != NULL)
+		if (uris != NULL && ephy_node_get_is_drag_dest (node))
 		{
 			/* FIXME fill success */
 			g_signal_emit (G_OBJECT (view),
@@ -416,12 +423,6 @@ drag_data_received_cb (GtkWidget *widget,
 			gtk_tree_path_free (path);
 		}
 	}
-
-
-	/* appease GtkTreeView by preventing its drag_data_receive
-	 * from being called */
-	g_signal_stop_emission_by_name (GTK_TREE_VIEW (view),
-					"drag_data_received");
 }
 
 static gboolean
@@ -662,6 +663,42 @@ did_not_drag (EphyNodeView *view,
         }                                                                                                              
 }
 
+typedef struct
+{
+	EphyNodeView *view;
+	gboolean result;
+}
+ForeachData;
+
+static void
+check_node_is_drag_source (GtkTreeModel *model,
+			   GtkTreePath *path,
+			   GtkTreeIter *iter,
+			   ForeachData *data)
+{
+	EphyNode *node;
+
+	node = get_node_from_path (data->view, path);
+	data->result = data->result &&
+		       node != NULL &&
+		       ephy_node_get_is_drag_source (node);
+}
+
+static gboolean
+can_drag_selection (EphyNodeView *view)
+{
+	GtkTreeView *tree_view = GTK_TREE_VIEW (view);
+	GtkTreeSelection *selection;
+	ForeachData data = { view, TRUE };
+
+	selection = gtk_tree_view_get_selection (tree_view);
+	gtk_tree_selection_selected_foreach (selection,
+					     (GtkTreeSelectionForeachFunc) check_node_is_drag_source,
+					     &data);
+
+	return data.result;
+}
+
 static void
 drag_data_get_cb (GtkWidget *widget,
                   GdkDragContext *context,
@@ -706,6 +743,7 @@ button_release_cb (GtkWidget *widget,
                         did_not_drag (view, event);
                         return TRUE;
                 }
+		view->priv->drag_started = FALSE;
         }
         return FALSE;
 }
@@ -726,7 +764,8 @@ motion_notify_cb (GtkWidget *widget,
 	{
 		if (gtk_drag_check_threshold (widget, view->priv->drag_x,
                                               view->priv->drag_y, event->x,
-					      event->y))
+					      event->y)
+		    && can_drag_selection (view))
 		{
 			context = gtk_drag_begin
 				(widget, view->priv->source_target_list,
