@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-toolbar.c,v 1.7.2.1 2005/03/11 20:32:48 chpe Exp $
+ *  $Id: ephy-toolbar.c,v 1.16 2005/08/08 20:19:56 chpe Exp $
  */
 
 #include "config.h"
@@ -35,8 +35,8 @@
 #include "ephy-dnd.h"
 #include "ephy-shell.h"
 #include "ephy-stock-icons.h"
+#include "ephy-action-helper.h"
 #include "window-commands.h"
-#include "eel-gconf-extensions.h"
 #include "ephy-debug.h"
 
 #include <glib/gi18n.h>
@@ -58,6 +58,11 @@ enum
 	LAST_ACTION
 };
 
+enum
+{
+	SENS_FLAG = 1 << 0
+};
+
 #define EPHY_TOOLBAR_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_TOOLBAR, EphyToolbarPrivate))
 
 struct _EphyToolbarPrivate
@@ -65,29 +70,26 @@ struct _EphyToolbarPrivate
 	EphyWindow *window;
 	GtkActionGroup *action_group;
 	GtkAction *actions[LAST_ACTION];
-	gboolean updating_address;
 	GtkWidget *fixed_toolbar;
 	GtkWidget *spinner;
 	GtkToolItem *spinner_item;
 	GtkToolItem *sep_item;
 	GtkToolItem *exit_button;
-	guint disable_arbitrary_url_notifier_id;
 	gulong set_focus_handler;
+	gboolean updating_address;
 	gboolean show_lock;
 	gboolean lock_visible;
 	gboolean leave_fullscreen_visible;
 	gboolean spinning;
 };
 
-static GtkTargetEntry drag_targets[] =
+static const GtkTargetEntry drag_targets [] =
 {
 	{ EGG_TOOLBAR_ITEM_TYPE, GTK_TARGET_SAME_APP, 0 },
-	{ EPHY_DND_TOPIC_TYPE,   0,		   1 },
+	{ EPHY_DND_TOPIC_TYPE,   0,		      1 },
 	{ EPHY_DND_URL_TYPE,	 0,		      2 }
 };
 
-#define CONF_LOCKDOWN_DISABLE_ARBITRARY_URL  "/apps/epiphany/lockdown/disable_arbitrary_url"
-
 enum
 {
 	PROP_0,
@@ -203,32 +205,6 @@ maybe_finish_activation_cb (EphyWindow *window,
 	}
 }
 
-static void
-update_location_editable (EphyToolbar *toolbar)
-{
-	EphyToolbarPrivate *priv = toolbar->priv;
-	EphyEmbed *embed;
-	char *address;
-	gboolean editable;
-
-	editable = !eel_gconf_get_boolean (CONF_LOCKDOWN_DISABLE_ARBITRARY_URL);
-
-	g_object_set (priv->actions[LOCATION_ACTION], "editable", editable, NULL);
-
-	/* Restore the real web page address when disabling entry */
-	if (editable == FALSE)
-	{
-		embed = ephy_window_get_active_embed (priv->window);
-		/* embed is NULL on startup */
-		if (EPHY_IS_EMBED (embed))
-		{
-			address = ephy_embed_get_location (embed, TRUE);
-			ephy_toolbar_set_location (toolbar, address);
-			g_free (address);
-		}
-	}
-}
-
 static void
 sync_user_input_cb (EphyLocationAction *action,
 		    GParamSpec *pspec,
@@ -248,7 +224,7 @@ sync_user_input_cb (EphyLocationAction *action,
 	address = ephy_location_action_get_address (action);
 
 	priv->updating_address = TRUE;
-	ephy_tab_set_location (tab, address, EPHY_TAB_ADDRESS_EXPIRE_CURRENT);
+	ephy_tab_set_typed_address (tab, address);
 	priv->updating_address = FALSE;
 }
 
@@ -267,15 +243,6 @@ zoom_to_level_cb (GtkAction *action,
 	ephy_window_set_zoom (toolbar->priv->window, zoom);
 }
 
-static void
-arbitrary_url_notifier (GConfClient *client,
-			guint cnxn_id,
-			GConfEntry *entry,
-			EphyToolbar *toolbar)
-{
-	update_location_editable (toolbar);
-}
-
 static void
 ephy_toolbar_set_window (EphyToolbar *toolbar,
 			 EphyWindow *window)
@@ -356,7 +323,7 @@ ephy_toolbar_set_window (EphyToolbar *toolbar,
 			      "name", "Location",
 			      "label", _("Address Entry"),
 			      "stock_id", EPHY_STOCK_ENTRY,
-			      "tooltip", _("Enter a web address to open, or a phrase to search for on the web"),
+			      "tooltip", _("Enter a web address to open, or a phrase to search for"),
 			      "visible-overflown", FALSE,
 			      "window", priv->window,
 			      NULL);
@@ -367,7 +334,6 @@ ephy_toolbar_set_window (EphyToolbar *toolbar,
 	g_signal_connect (action, "lock-clicked",
 			  G_CALLBACK (lock_clicked_cb), toolbar);
 	gtk_action_group_add_action (priv->action_group, action);
-	update_location_editable (toolbar);
 	g_object_unref (action);
 
 	action = priv->actions[ZOOM_ACTION] =
@@ -405,17 +371,13 @@ ephy_toolbar_set_window (EphyToolbar *toolbar,
 				  G_CALLBACK (ephy_link_open), toolbar);
 	gtk_action_group_add_action_with_accel (priv->action_group, action, "<alt>Home");
 	g_object_unref (action);
-
-	priv->disable_arbitrary_url_notifier_id = eel_gconf_notification_add
-		(CONF_LOCKDOWN_DISABLE_ARBITRARY_URL,
-		 (GConfClientNotifyFunc) arbitrary_url_notifier, toolbar);
 }
 
 /* public functions */
 
 void
 ephy_toolbar_set_favicon (EphyToolbar *toolbar,
-			  const char *icon)
+			  GdkPixbuf *icon)
 {
 	EphyToolbarPrivate *priv = toolbar->priv;
 
@@ -448,10 +410,13 @@ ephy_toolbar_activate_location (EphyToolbar *toolbar)
 		entry = GTK_WIDGET (proxies->data);
 	}
 
-	/* happens when the user has removed the location entry from
-	  * the toolbars, or when it's overflown
-	  */
-	if (entry == NULL || !GTK_WIDGET_REALIZED (entry)) return;
+	if (entry == NULL)
+	{
+		/* happens when the user has removed the location entry from
+		 * the toolbars.
+		 */
+		return;
+	}
 
 	g_object_get (G_OBJECT (toolbar), "visible", &visible, NULL);
 	if (visible == FALSE)
@@ -477,15 +442,16 @@ ephy_toolbar_get_location (EphyToolbar *toolbar)
 
 void
 ephy_toolbar_set_location (EphyToolbar *toolbar,
-			   const char *address)
+			   const char *address,
+			   const char *typed_address)
 {
 	EphyToolbarPrivate *priv = toolbar->priv;
 	EphyLocationAction *action = EPHY_LOCATION_ACTION (priv->actions[LOCATION_ACTION]);
-	
+
 	if (priv->updating_address) return;
 
 	priv->updating_address = TRUE;
-	ephy_location_action_set_address (action, address);
+	ephy_location_action_set_address (action, address, typed_address);
 	priv->updating_address = FALSE;
 }
 
@@ -497,9 +463,9 @@ ephy_toolbar_set_navigation_actions (EphyToolbar *toolbar,
 {
 	EphyToolbarPrivate *priv = toolbar->priv;
 
-	g_object_set (priv->actions[BACK_ACTION], "sensitive", back, NULL);
-	g_object_set (priv->actions[FORWARD_ACTION], "sensitive", forward, NULL);
-	g_object_set (priv->actions[UP_ACTION], "sensitive", up, NULL);
+	ephy_action_change_sensitivity_flags (priv->actions[BACK_ACTION], SENS_FLAG, !back);
+	ephy_action_change_sensitivity_flags (priv->actions[FORWARD_ACTION], SENS_FLAG, !forward);
+	ephy_action_change_sensitivity_flags (priv->actions[UP_ACTION], SENS_FLAG, !up);
 }
 
 void
@@ -550,10 +516,8 @@ ephy_toolbar_set_zoom (EphyToolbar *toolbar,
 {
 	EphyToolbarPrivate *priv = toolbar->priv;
 
-	g_object_set (priv->actions[ZOOM_ACTION],
-		      "zoom", can_zoom ? zoom : 1.0,
-		      "sensitive", can_zoom,
-		      NULL);
+	gtk_action_set_sensitive (priv->actions[ZOOM_ACTION], can_zoom);
+	g_object_set (priv->actions[ZOOM_ACTION], "zoom", can_zoom ? zoom : 1.0, NULL);
 }
 
 /* Class implementation */
@@ -686,8 +650,6 @@ ephy_toolbar_finalize (GObject *object)
 					     priv->set_focus_handler);
 	}
 
-	eel_gconf_notification_remove (priv->disable_arbitrary_url_notifier_id);
-
 	g_signal_handlers_disconnect_by_func
 		(egg_editable_toolbar_get_model (etoolbar),
 		 G_CALLBACK (toolbar_added_cb), toolbar);
